% Purpose: This is an implementation of the simple gradient method for 
%          convex optimization problems.
% Created by Quoc Tran-Dinh, STOR892, UNC-Chapel Hill.
%
function [xopt, output] = non_acc_grad_alg(fx, gradfx, L, x0, maxiters, tol)
    
    % Check the inputs.
    if nargin < 6,        tol      = 1.0e-3; end
    if nargin < 5,        maxiters = 500;    end
    if isempty(tol),      tol      = 1.0e-3; end
    if isempty(maxiters), maxiters = 500;    end
    if nargin < 4, error('It requires at least 4 inputs!'); end
        
    % Initialization.
    fprintf('****** Simple gradient method *****\n');
    time1 = tic;
    x_cur = x0;
    
    % The main loop.
    for iter = 1:maxiters
        
        % Evaluate the objective gradient and value.
        grad_fx_cur = gradfx(x_cur);
        fx_val      = fx(x_cur);
        
        % Update the next iteration.
        x_next  = x_cur - (1/L)*grad_fx_cur;
        
        % Compute the error
        abs_err = norm(x_next - x_cur, 2);
        rel_err = abs_err/norm(x_cur, 2);
        
        % Print and store the iteration.
        fprintf('Iter = %3d, Error = %3.3e, F(x) = %3.3e\n', iter, rel_err, fx_val);
        output.hist.fx(iter,1)      = fx_val;
        output.hist.rel_err(iter,1) = rel_err;
        
        % Check the stopping criterion.
        if rel_err <= tol
            output.msg = 'Convergence achieved';
            break;
        end
        
        % Move to the next loop.
        x_cur = x_next;
        
    end
    % End of the main loop.
    
    % Finalization.
    if iter >= maxiters
        output.msg = 'Exceed the maximum number of iterations';
    end
    xopt           = x_cur;
    output.iter    = iter;
    output.time    = toc(time1);
    output.fx      = fx_val;
    output.rel_err = rel_err;
end